@@ -7,7 +7,11 @@
 ;; notice and this notice are preserved.  This file is offered as-is,
 ;; without any warranty.
 
+(add-to-load-path "../../../../module")
+(add-to-load-path "../../../../examples")
+
 (use-modules (nyacc lang javascript mach))
+;;(use-modules (nyacc lang javascript parser))
 (use-modules (nyacc lang javascript pprint))
 (use-modules (nyacc lalr))
 (use-modules (nyacc parse))
@@ -29,11 +33,11 @@
       (pp-lalr-grammar js-spec)
       (pp-lalr-machine js-mach))))
 
-(when #t
+(when #f
   (with-output-to-file "gram.y.new"
     (lambda () (lalr->bison js-spec))))
 
-(when #t
+(when #f
   (let ((res (with-input-from-file "ex1.js" dev-parse-js)))
     (pretty-print res)
     (pretty-print-js res)))
