@@ -18,7 +18,7 @@
 ;; C parser
 
 (define-module (nyacc lang c99 xparser)
-  #:export (parse-cx parse-c99x)
+  #:export (parse-c99x)
   #:use-module (nyacc lex)
   #:use-module (nyacc parse)
   #:use-module (nyacc lang util)
@@ -45,13 +45,13 @@
   (let ((info (fluid-ref *info*)))
     (raw-parser (gen-c-lexer) #:debug (cpi-debug info))))
 
-;; @item parse-c99x [#:cpp-defs def-a-list] [#:debug bool]
+;; @item parse-c99x [#:cpp-defs defs] [#:debug bool]
 ;; This needs to be explained in some detail.
 ;; [#:tyns '("foo_t")]
 (define* (parse-c99x expr-string
 		     #:key
 		     (cpp-defs '())	; CPP defines
-		     (tn-dict '())	; typedef dictionary
+		     (inc-help '())	; include helper
 		     (xdef? #f)		; pred to determine expand
 		     (debug #f)		; debug?
 		     (tyns '()))	; defined typenames
@@ -60,7 +60,7 @@
       (catch
        'c99-error
        (lambda ()
-	 (let ((info (make-cpi debug cpp-defs '(".") tn-dict)))
+	 (let ((info (make-cpi debug cpp-defs '(".") inc-help)))
 	   (set-cpi-ptl! info (cons tyns (cpi-ptl info)))
 	   (with-fluid*
 	       *info* info
@@ -71,6 +71,4 @@
 	 (report-error fmt rest)
 	 #f)))))
 
-(define parse-cx parse-c99x)
-
 ;; --- last line ---
